<?php

namespace Drupal\entity_template_ui\Plugin\EntityTemplate\Template;

use Drupal\Core\Url;

class InlineTemplateUI extends BaseTemplateUI {

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\SubTemplateInterface
   */
  protected $template;

  /**
   * {@inheritdoc}
   */
  public function addComponentUrl(): ?Url {
    // @todo: Don't assume that the root template is a blueprint template.
    /** @var \Drupal\entity_template\Plugin\EntityTemplate\Template\BlueprintTemplateInterface $root_template */
    $root_template = $this->template->getRootTemplate();

    return Url::fromRoute(
      'entity_template_ui.edit_template.choose_component.sub',
      [
        'template_blueprint_provider' => $root_template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $root_template->getBlueprint()->getProviderKey(),
        'template_id' => $root_template->getKey(),
        'sub_template' => $this->template->getPath()
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function swapComponentUrl(string $key, string $other_id): ?Url {
    // @todo: Don't assume that the root template is a blueprint template.
    /** @var \Drupal\entity_template\Plugin\EntityTemplate\Template\BlueprintTemplateInterface $root_template */
    $root_template = $this->template->getRootTemplate();

    return Url::fromRoute(
      'entity_template_ui.edit_template.swap_component.sub',
      [
        'template_blueprint_provider' => $root_template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $root_template->getBlueprint()->getProviderKey(),
        'template_key' => $root_template->getKey(),
        'sub_template' => $this->template->getPath(),
        'component_id' => $key,
        'other_plugin' => $other_id,
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function removeComponentUrl(string $key): ?Url {
    $root_template = $this->template->getRootTemplate();

    return Url::fromRoute(
      'entity_template_ui.edit_template.remove_component.sub',
      [
        'template_blueprint_provider' => $root_template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $root_template->getBlueprint()->getProviderKey(),
        'template_key' => $root_template->getKey(),
        'sub_template' => $this->template->getPath(),
        'component_id' => $key,
      ]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function dataSelectAutocompleteElement(string $component_id): array {
    $root_template = $this->template->getRootTemplate();

    return [
      '#autocomplete_route_name' => 'entity_template_ui.edit_template.component.dsautocomplete.sub',
      '#autocomplete_route_parameters' => [
        'template_blueprint_provider' => $root_template->getBlueprint()->getProvider()->getPluginId(),
        'blueprint_storage' => $root_template->getBlueprint()->getProviderKey(),
        'template_key' =>  $root_template->getKey(),
        'sub_template' => $this->template->getPath(),
        'component_id' => $component_id,
      ],
    ];
  }
}
