<?php

namespace Drupal\entity_template;

use Drupal\Core\Url;

trait BlueprintStorageTrait {

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider\BlueprintProviderInterface
   */
  protected $provider;

  /**
   * Get the blueprint provider for this blueprint storage.
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider\BlueprintProviderInterface
   */
  public function getProvider() {
    return $this->provider;
  }

  /**
   * {@inheritdoc}
   */
  public function getProviderKey() {
    if ($this instanceof BlueprintStorageInterface) {
      return $this->getProvider()->getBlueprintKey($this);
    }

    return 'invalid';
  }

  /**
   * Get the tempstore key.
   *
   * @return string
   */
  public function getTempstoreKey() {
    return $this->getProviderKey();
  }

  /**
   * Get the collection of templates.
   *
   * @return \Drupal\Component\Plugin\LazyPluginCollection
   */
  abstract public function getTemplates();

  /**
   * {@inheritdoc}
   */
  public function addTemplate(array $configuration) {
    $configuration['uuid'] = $this->uuidGenerator()->generate();
    $this->getTemplates()->addInstanceId($configuration['uuid'], $configuration);
    return $configuration['uuid'];
  }

  /**
   * {@inheritdoc}
   */
  public function removeTemplate($id) {
    $this->getTemplates()->removeInstanceId($id);
  }

  /**
   * Get the uuid service.
   *
   * @return \Drupal\Component\Uuid\UuidInterface
   */
  protected function uuidGenerator() {
    return \Drupal::service('uuid');
  }

  /**
   * Get the edit url for this blueprint.
   *
   * @return \Drupal\Core\Url
   */
  public function getEditUrl() {
    return $this->getProvider()->getBlueprintEditUrl($this);
  }

  /**
   * @param string $key
   *
   * @return \Drupal\Core\Url
   */
  public function getEditTemplateUrl(string $key): Url {
    return $this->getProvider()->getBlueprintEditTemplateUrl($this, $key);
  }
}
