<?php

namespace Drupal\entity_template\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Class TemplateBuilder
 *
 * @ConfigEntityType(
 *   id = "entity_template_builder",
 *   label = @Translation("Template Builder"),
 *   admin_permission = "administer template builders",
 *   config_prefix = "builder",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "default_blueprint",
 *     "return_type",
 *     "parameters",
 *   },
 * )
 *
 * @package Drupal\entity_template\Entity
 */
class TemplateBuilder extends ConfigEntityBase {

  /**
   * @var string
   */
  protected $id;

  /**
   * @var string
   */
  protected $label;

  /**
   * @var array
   */
  protected $parameters = [];

  /**
   * @var \Drupal\entity_template\BlueprintInterface
   */
  protected $default_blueprint;

  /**
   * @var string
   */
  protected $return_type;

  /**
   * The parameters this builder passes.
   *
   * @return array[]
   *   Each array has:
   *     - machine_name
   *     - type
   *     - label
   */
  public function getParameters() {
    return $this->parameters;
  }

  /**
   * Add a parameter
   *
   * @param $machine_name
   * @param $configuration
   */
  public function addParameter($machine_name, $configuration) {
    $this->parameters[$machine_name] = $configuration;
  }

  /**
   * Remove a parameter
   *
   * @param $machine_name
   */
  public function removeParameter($machine_name) {
    unset($this->parameters[$machine_name]);
  }

  /**
   * @param $machine_name
   *
   * @return bool
   */
  public function hasParameter($machine_name) {
    return isset($this->parameters[$machine_name]);
  }

  /**
   * The return type this builder expects to produce.
   *
   * @return string
   */
  public function getReturnType() {
    return $this->return_type;
  }
}
