<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\Builder;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Plugin\ContextAwarePluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\entity_template\BlueprintStorageInterface;
use Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider\BlueprintProviderInterface;

/**
 * Interface BuilderInterface
 *
 * @package Drupal\entity_template\Plugin\EntityTemplate\Builder
 */
interface BuilderInterface extends PluginInspectionInterface, ContextAwarePluginInterface {

  /**
   * Does the builder provide the given context.
   *
   * @return boolean
   */
  public function hasContextDefinition($name);

  /**
   * Get the default blueprint for this builder.
   *
   * @return \Drupal\entity_template\BlueprintInterface|null
   */
  public function getDefaultBlueprint();

  /**
   * Get the default blueprint storage.
   *
   * @param \Drupal\entity_template\Plugin\EntityTemplate\BlueprintProvider\BlueprintProviderInterface $provider
   *
   * @return \Drupal\entity_template\BlueprintStorageInterface|null
   */
  public function getDefaultBlueprintStorage(BlueprintProviderInterface $provider);

  /**
   * Get the default blueprint edit template url.
   *
   * @param \Drupal\entity_template\BlueprintStorageInterface $blueprint_storage
   * @param string $key
   *
   * @return \Drupal\Core\Url
   */
  public function getDefaultBlueprintEditTemplateUrl(BlueprintStorageInterface $blueprint_storage, string $key): Url;

  /**
   * The type of state this returns
   *
   * @return string
   *   A typed data data type name, optionally with [] on the end to specify a
   *   list.
   */
  public function getReturnType();

  /**
   * Get whether the builder returns a list.
   *
   * @return bool
   */
  public function returnsList();

  /**
   * Get the available blueprints for this builder.
   *
   * @param array $parameters
   * @param \Drupal\Core\Session\AccountInterface|NULL $account
   *
   * @return \Drupal\entity_template\BlueprintInterface[]
   */
  public function getAvailableBlueprints(array $parameters = [], AccountInterface $account = NULL);

  /**
   * Execute the builder.
   *
   * @param array $parameters
   * @param \Drupal\Core\Session\AccountInterface|NULL $account
   * @param string $blueprint_key
   *
   * @return \Drupal\entity_template\BlueprintResult
   *
   * @throws \InvalidArgumentException
   *   If an invalid blueprint key is provided.
   * @throws \Drupal\entity_template\Exception\MultipleAvailableBlueprintException
   *   If the builder can't decide on a best blueprint to use.
   * @throws \Drupal\entity_template\Exception\NoAvailableBlueprintException
   *   If the builder hasn't got any available blueprints.
   */
  public function execute(array $parameters = [], AccountInterface $account = NULL, $blueprint_key = NULL);

  /**
   * Get the admin operations.
   *
   * @return array
   */
  public function getAdminOperations();

}
