<?php

namespace Drupal\entity_template\Plugin\EntityTemplate\Template;

trait SubTemplateTrait {

  /**
   * Get the parent template.
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  abstract public function getParentTemplate() : ?TemplateInterface;

  /**
   * Get the key of this template.
   *
   * @return string
   */
  abstract public function getKey() : string;

  /**
   * Get the root template.
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  public function getRootTemplate() {
    $parent = $this;
    do {
      if (!($parent instanceof SubTemplateInterface) || !$parent->getParentTemplate()) {
        return $parent;
      }
    } while ($parent = $parent->getParentTemplate());

    return $parent;
  }

  /**
   * Get the template that provides the contexts.
   *
   * @return \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  public function getContextProvidingTemplate() : TemplateInterface{
    /** @var \Drupal\entity_template\Plugin\EntityTemplate\Template\SubTemplateInterface $template */
    $template = $this;

    do {
      if (is_callable([$template, 'getContexts'])) {
        return $template;
      }
    } while ($template instanceof SubTemplateInterface && $template = $template->getParentTemplate());

    return $template;
  }

  /**
   * {@inheritdoc}
   */
  public function getTemplateContexts($sample = FALSE) {
    return $this->getContextProvidingTemplate()->getTemplateContexts($sample);
  }

  /**
   * Get the path.
   *
   * @return string
   */
  public function getPath() {
    $template = $this;
    $path = $template->getKey();

    while (
      $template instanceof  SubTemplateInterface &&
      ($template = $template->getParentTemplate()) &&
      $template instanceof SubTemplateInterface
    ) {
      $path = $template->getKey().':'.$path;
    }

    return $path;
  }


}
