<?php

namespace Drupal\entity_template;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Plugin\DefaultLazyPluginCollection;
use Drupal\entity_template\Plugin\EntityTemplate\Component\TemplateContextAwareComponentInterface;
use Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface;

/**
 * Class TemplateLazyComponentsCollection
 *
 * @package Drupal\entity_template
 */
class TemplateLazyComponentsCollection extends DefaultLazyPluginCollection {

  /**
   * @var \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface
   */
  protected $template;

  /**
   * TemplateLazyComponentsCollection constructor.
   *
   * @param \Drupal\entity_template\Plugin\EntityTemplate\Template\TemplateInterface $template
   * @param \Drupal\Component\Plugin\PluginManagerInterface $manager
   * @param array $configurations
   */
  public function __construct(TemplateInterface $template, PluginManagerInterface $manager, array $configurations = []) {
    $this->template = $template;

    parent::__construct($manager, $configurations);
  }

  /**
   * {@inheritdoc}
   */
  protected function initializePlugin($instance_id) {
    $configuration = isset($this->configurations[$instance_id]) ? $this->configurations[$instance_id] : [];
    if (!isset($configuration[$this->pluginKey])) {
      throw new PluginNotFoundException($instance_id);
    }

    $instance = $this->manager->createInstance(
      $configuration[$this->pluginKey],
      $configuration
    );
    if ($instance instanceof TemplateContextAwareComponentInterface) {
      $instance->setTemplate($this->template);
    }

    $this->set($instance_id, $instance);
  }
}
